@@ -1,6 +1,6 @@
 package Git::Raw::Repository;
 {
-  $Git::Raw::Repository::VERSION = '0.17';
+  $Git::Raw::Repository::VERSION = '0.18';
 }
 
 use strict;
@@ -14,7 +14,25 @@ Git::Raw::Repository - Git repository class
 
 =head1 VERSION
 
-version 0.17
+version 0.18
+
+=head1 SYNOPSIS
+
+    use Git::Raw;
+
+    # create a new dangling remote
+    my $url    = 'git://github.com/ghedo/p5-Git-Raw.git';
+    my $origin = Git::Raw::Remote -> new(undef, 'origin', $url);
+
+    # clone the Git repository
+    my $repo = Git::Raw::Repository -> clone(
+      $origin, 'p5-Git-Raw', { 'update_missing' => 1}, 0
+    );
+
+    # print all the tags of the repository
+    foreach my $tag (@{ $repo -> tags }) {
+      say $tag -> name;
+    }
 
 =head1 DESCRIPTION
 
@@ -26,10 +44,11 @@ A C<Git::Raw::Repository> represents a Git repository.
 
 Initialize a new repository at C<$path>.
 
-=head2 clone( $url, $path, \%strategy, $is_bare )
+=head2 clone( $remote, $path, \%strategy, $is_bare )
 
-Clone the repository at C<$url> to C<$path>. See the C<checkout()> description
-for more information about C<%strategy>.
+Clone the repository at C<$remote> to C<$path>. See the C<checkout()> description
+for more information about C<%strategy>. The C<$remote> object must be created
+passing an C<undef> repository.
 
 =head2 open( $path )
 
