@@ -203,21 +203,25 @@ static int checkout_blob(
 	return error;
 }
 
-static int retrieve_symlink_caps(git_repository *repo, bool *can_symlink)
+static int retrieve_symlink_caps(git_repository *repo, bool *out)
 {
 	git_config *cfg;
+	int can_symlink = 0;
 	int error;
 
 	if (git_repository_config__weakptr(&cfg, repo) < 0)
 		return -1;
 
-	error = git_config_get_bool((int *)can_symlink, cfg, "core.symlinks");
+	error = git_config_get_bool(&can_symlink, cfg, "core.symlinks");
 
 	/* If "core.symlinks" is not found anywhere, default to true. */
 	if (error == GIT_ENOTFOUND) {
-		*can_symlink = true;
+		can_symlink = true;
 		error = 0;
 	}
+        
+	if (error >= 0)
+		*out = can_symlink;
 
 	return error;
 }
@@ -228,7 +232,7 @@ static void normalize_options(
 	assert(normalized);
 
 	if (!proposed)
-		memset(normalized, 0, sizeof(git_checkout_opts));
+		GIT_INIT_STRUCTURE(normalized, GIT_CHECKOUT_OPTS_VERSION);
 	else
 		memmove(normalized, proposed, sizeof(git_checkout_opts));
 
@@ -444,8 +448,7 @@ static int checkout_get_actions(
 		!(error == GIT_ENOTFOUND || error == GIT_EORPHANEDHEAD))
 			return -1;
 
-	if ((error = git_iterator_for_tree_range(
-			 &hiter, data->repo, head, pfx, pfx)) < 0)
+	if ((error = git_iterator_for_tree_range(&hiter, head, pfx, pfx)) < 0)
 		goto fail;
 
 	if ((diff->opts.flags & GIT_DIFF_DELTAS_ARE_ICASE) != 0 &&
@@ -607,7 +610,7 @@ int git_checkout_index(
 	git_checkout_opts *opts)
 {
 	git_diff_list *diff = NULL;
-	git_diff_options diff_opts = {0};
+	git_diff_options diff_opts = GIT_DIFF_OPTIONS_INIT;
 	git_checkout_opts checkout_opts;
 	checkout_diff_data data;
 	git_buf workdir = GIT_BUF_INIT;
@@ -617,6 +620,8 @@ int git_checkout_index(
 
 	assert(repo);
 
+	GITERR_CHECK_VERSION(opts, GIT_CHECKOUT_OPTS_VERSION, "git_checkout_opts");
+
 	if ((error = git_repository__ensure_not_bare(repo, "checkout")) < 0)
 		return error;
 
