@@ -57,30 +57,69 @@ static int download_tags_value(git_remote *remote, git_config *cfg)
 	return error;
 }
 
+static int ensure_remote_name_is_valid(const char *name)
+{
+	git_buf buf = GIT_BUF_INIT;
+	git_refspec refspec;
+	int error = -1;
+
+	if (!name || *name == '\0')
+		goto cleanup;
+
+	git_buf_printf(&buf, "refs/heads/test:refs/remotes/%s/test", name);
+	error = git_refspec__parse(&refspec, git_buf_cstr(&buf), true);
+
+	git_buf_free(&buf);
+	git_refspec__free(&refspec);
+
+cleanup:
+	if (error) {
+		giterr_set(
+			GITERR_CONFIG,
+			"'%s' is not a valid remote name.", name);
+		error = GIT_EINVALIDSPEC;
+	}
+
+	return error;
+}
+
 int git_remote_new(git_remote **out, git_repository *repo, const char *name, const char *url, const char *fetch)
 {
 	git_remote *remote;
+	git_buf fetchbuf = GIT_BUF_INIT;
+	int error = -1;
 
 	/* name is optional */
-	assert(out && repo && url);
+	assert(out && url);
 
-	remote = git__malloc(sizeof(git_remote));
+	remote = git__calloc(1, sizeof(git_remote));
 	GITERR_CHECK_ALLOC(remote);
 
-	memset(remote, 0x0, sizeof(git_remote));
 	remote->repo = repo;
 	remote->check_cert = 1;
 	remote->update_fetchhead = 1;
 
 	if (git_vector_init(&remote->refs, 32, NULL) < 0)
-		return -1;
+		goto on_error;
 
 	remote->url = git__strdup(url);
 	GITERR_CHECK_ALLOC(remote->url);
 
 	if (name != NULL) {
+		if ((error = ensure_remote_name_is_valid(name)) < 0) {
+			error = GIT_EINVALIDSPEC;
+			goto on_error;
+		}
+
 		remote->name = git__strdup(name);
 		GITERR_CHECK_ALLOC(remote->name);
+
+		/* An empty name indicates to use a sensible default for the fetchspec. */
+		if (fetch && !(*fetch)) {
+			if (git_buf_printf(&fetchbuf, "+refs/heads/*:refs/remotes/%s/*", remote->name) < 0)
+				goto on_error;
+			fetch = git_buf_cstr(&fetchbuf);
+		}
 	}
 
 	if (fetch != NULL) {
@@ -94,11 +133,26 @@ int git_remote_new(git_remote **out, git_repository *repo, const char *name, con
 	}
 
 	*out = remote;
+	git_buf_free(&fetchbuf);
 	return 0;
 
 on_error:
 	git_remote_free(remote);
-	return -1;
+	git_buf_free(&fetchbuf);
+	return error;
+}
+
+int git_remote_set_repository(git_remote *remote, git_repository *repo)
+{
+	assert(repo);
+
+	if (remote->repo) {
+		giterr_set(GITERR_INVALID, "Remotes can't change repositiories.");
+		return GIT_ERROR;
+	}
+
+	remote->repo = repo;
+	return 0;
 }
 
 int git_remote_load(git_remote **out, git_repository *repo, const char *name)
@@ -111,6 +165,9 @@ int git_remote_load(git_remote **out, git_repository *repo, const char *name)
 
 	assert(out && repo && name);
 
+	if ((error = ensure_remote_name_is_valid(name)) < 0)
+		return error;
+
 	if (git_repository_config__weakptr(&config, repo) < 0)
 		return -1;
 
@@ -212,30 +269,6 @@ cleanup:
 	return error;
 }
 
-static int ensure_remote_name_is_valid(const char *name)
-{
-	git_buf buf = GIT_BUF_INIT;
-	git_refspec refspec;
-	int error = -1;
-
-	if (!name || *name == '\0')
-		goto cleanup;
-
-	git_buf_printf(&buf, "refs/heads/test:refs/remotes/%s/test", name);
-	error = git_refspec__parse(&refspec, git_buf_cstr(&buf), true);
-
-	git_buf_free(&buf);
-	git_refspec__free(&refspec);
-
-cleanup:
-	if (error)
-		giterr_set(
-			GITERR_CONFIG,
-			"'%s' is not a valid remote name.", name);
-
-	return error;
-}
-
 static int update_config_refspec(
 	git_config *config,
 	const char *remote_name,
@@ -279,8 +312,13 @@ int git_remote_save(const git_remote *remote)
 
 	assert(remote);
 
-	if (ensure_remote_name_is_valid(remote->name) < 0)
-		return -1;
+	if (!remote->repo) {
+		giterr_set(GITERR_INVALID, "Can't save a dangling remote.");
+		return GIT_ERROR;
+	}
+
+	if ((error = ensure_remote_name_is_valid(remote->name)) < 0)
+		return error;
 
 	if (git_repository_config__weakptr(&config, remote->repo) < 0)
 		return -1;
@@ -498,7 +536,7 @@ int git_remote_connect(git_remote *remote, git_direction direction)
 	if (!remote->check_cert)
 		flags |= GIT_TRANSPORTFLAGS_NO_CHECK_CERT;
 
-	if (t->connect(t, url, remote->cred_acquire_cb, direction, flags) < 0)
+	if (t->connect(t, url, remote->cred_acquire_cb, remote->cred_acquire_payload, direction, flags) < 0)
 		goto on_error;
 
 	remote->transport = t;
@@ -533,7 +571,7 @@ int git_remote__get_http_proxy(git_remote *remote, bool use_ssl, char **proxy_ur
 
 	assert(remote);
 
-	if (!proxy_url)
+	if (!proxy_url || !remote->repo)
 		return -1;
 
 	*proxy_url = NULL;
@@ -735,6 +773,11 @@ int git_remote_update_tips(git_remote *remote)
 
 	assert(remote);
 
+	if (!remote->repo) {
+		giterr_set(GITERR_INVALID, "Can't update tips on a dangling remote.");
+		return GIT_ERROR;
+	}
+
 	spec = &remote->fetch;
 	
 	if (git_repository_odb__weakptr(&odb, remote->repo) < 0)
@@ -958,6 +1001,10 @@ int git_remote_list(git_strarray *remotes_list, git_repository *repo)
 int git_remote_add(git_remote **out, git_repository *repo, const char *name, const char *url)
 {
 	git_buf buf = GIT_BUF_INIT;
+	int error;
+
+	if ((error = ensure_remote_name_is_valid(name)) < 0)
+		return error;
 
 	if (git_buf_printf(&buf, "+refs/heads/*:refs/remotes/%s/*", name) < 0)
 		return -1;
@@ -985,10 +1032,12 @@ void git_remote_check_cert(git_remote *remote, int check)
 	remote->check_cert = check;
 }
 
-void git_remote_set_callbacks(git_remote *remote, git_remote_callbacks *callbacks)
+int git_remote_set_callbacks(git_remote *remote, git_remote_callbacks *callbacks)
 {
 	assert(remote && callbacks);
 
+	GITERR_CHECK_VERSION(callbacks, GIT_REMOTE_CALLBACKS_VERSION, "git_remote_callbacks");
+
 	memcpy(&remote->callbacks, callbacks, sizeof(git_remote_callbacks));
 
 	if (remote->transport && remote->transport->set_callbacks)
@@ -996,21 +1045,27 @@ void git_remote_set_callbacks(git_remote *remote, git_remote_callbacks *callback
 			remote->callbacks.progress,
 			NULL,
 			remote->callbacks.payload);
+
+	return 0;
 }
 
 void git_remote_set_cred_acquire_cb(
 	git_remote *remote,
-	git_cred_acquire_cb cred_acquire_cb)
+	git_cred_acquire_cb cred_acquire_cb,
+	void *payload)
 {
 	assert(remote);
 
 	remote->cred_acquire_cb = cred_acquire_cb;
+	remote->cred_acquire_payload = payload;
 }
 
 int git_remote_set_transport(git_remote *remote, git_transport *transport)
 {
 	assert(remote && transport);
 
+	GITERR_CHECK_VERSION(transport, GIT_TRANSPORT_VERSION, "git_transport");
+
 	if (remote->transport) {
 		giterr_set(GITERR_NET, "A transport is already bound to this remote");
 		return -1;
@@ -1271,50 +1326,52 @@ int git_remote_rename(
 
 	assert(remote && new_name);
 
-	if ((error = ensure_remote_doesnot_exist(remote->repo, new_name)) < 0)
-		return error;
-
 	if ((error = ensure_remote_name_is_valid(new_name)) < 0)
 		return error;
 
-	if (!remote->name) {
+	if (remote->repo) {
+		if ((error = ensure_remote_doesnot_exist(remote->repo, new_name)) < 0)
+			return error;
+
+		if (!remote->name) {
+			if ((error = rename_fetch_refspecs(
+				remote,
+				new_name,
+				callback,
+				payload)) < 0)
+				return error;
+
+			remote->name = git__strdup(new_name);
+
+			return git_remote_save(remote);
+		}
+
+		if ((error = rename_remote_config_section(
+			remote->repo,
+			remote->name,
+			new_name)) < 0)
+				return error;
+
+		if ((error = update_branch_remote_config_entry(
+			remote->repo,
+			remote->name,
+			new_name)) < 0)
+				return error;
+
+		if ((error = rename_remote_references(
+			remote->repo,
+			remote->name,
+			new_name)) < 0)
+				return error;
+
 		if ((error = rename_fetch_refspecs(
 			remote,
 			new_name,
 			callback,
 			payload)) < 0)
 			return error;
-
-		remote->name = git__strdup(new_name);
-
-		return git_remote_save(remote);
 	}
 
-	if ((error = rename_remote_config_section(
-		remote->repo,
-		remote->name,
-		new_name)) < 0)
-			return error;
-
-	if ((error = update_branch_remote_config_entry(
-		remote->repo,
-		remote->name,
-		new_name)) < 0)
-			return error;
-
-	if ((error = rename_remote_references(
-		remote->repo,
-		remote->name,
-		new_name)) < 0)
-			return error;
-
-	if ((error = rename_fetch_refspecs(
-		remote,
-		new_name,
-		callback,
-		payload)) < 0)
-		return error;
-
 	git__free(remote->name);
 	remote->name = git__strdup(new_name);
 
