@@ -12,7 +12,7 @@ void test_diff_blob__initialize(void)
 
 	g_repo = cl_git_sandbox_init("attr");
 
-	memset(&opts, 0, sizeof(opts));
+	GIT_INIT_STRUCTURE(&opts, GIT_DIFF_OPTIONS_VERSION);
 	opts.context_lines = 1;
 	opts.interhunk_lines = 0;
 
@@ -30,7 +30,10 @@ void test_diff_blob__initialize(void)
 void test_diff_blob__cleanup(void)
 {
 	git_blob_free(d);
+	d = NULL;
+
 	git_blob_free(alien);
+	alien = NULL;
 
 	cl_git_sandbox_cleanup();
 }
@@ -56,11 +59,11 @@ void test_diff_blob__can_compare_text_blobs(void)
 
 	/* diff on tests/resources/attr/root_test1 */
 	cl_git_pass(git_diff_blobs(
-		a, b, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		a, b, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_mods);
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_MODIFIED]);
+	cl_assert_equal_i(0, expected.files_binary);
 
 	cl_assert_equal_i(1, expected.hunks);
 	cl_assert_equal_i(6, expected.lines);
@@ -71,11 +74,11 @@ void test_diff_blob__can_compare_text_blobs(void)
 	/* diff on tests/resources/attr/root_test2 */
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		b, c, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		b, c, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_mods);
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_MODIFIED]);
+	cl_assert_equal_i(0, expected.files_binary);
 
 	cl_assert_equal_i(1, expected.hunks);
 	cl_assert_equal_i(15, expected.lines);
@@ -86,11 +89,11 @@ void test_diff_blob__can_compare_text_blobs(void)
 	/* diff on tests/resources/attr/root_test3 */
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		a, c, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		a, c, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_mods);
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_MODIFIED]);
+	cl_assert_equal_i(0, expected.files_binary);
 
 	cl_assert_equal_i(1, expected.hunks);
 	cl_assert_equal_i(13, expected.lines);
@@ -100,11 +103,11 @@ void test_diff_blob__can_compare_text_blobs(void)
 
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		c, d, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		c, d, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_mods);
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_MODIFIED]);
+	cl_assert_equal_i(0, expected.files_binary);
 
 	cl_assert_equal_i(2, expected.hunks);
 	cl_assert_equal_i(14, expected.lines);
@@ -122,11 +125,11 @@ void test_diff_blob__can_compare_against_null_blobs(void)
 	git_blob *e = NULL;
 
 	cl_git_pass(git_diff_blobs(
-		d, e, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		d, e, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_dels);
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_DELETED]);
+	cl_assert_equal_i(0, expected.files_binary);
 
 	cl_assert_equal_i(1, expected.hunks);
 	cl_assert_equal_i(14, expected.hunk_old_lines);
@@ -137,11 +140,11 @@ void test_diff_blob__can_compare_against_null_blobs(void)
 	memset(&expected, 0, sizeof(expected));
 
 	cl_git_pass(git_diff_blobs(
-		d, e, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		d, e, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_adds);
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_ADDED]);
+	cl_assert_equal_i(0, expected.files_binary);
 
 	cl_assert_equal_i(1, expected.hunks);
 	cl_assert_equal_i(14, expected.hunk_new_lines);
@@ -152,67 +155,65 @@ void test_diff_blob__can_compare_against_null_blobs(void)
 	memset(&expected, 0, sizeof(expected));
 
 	cl_git_pass(git_diff_blobs(
-		alien, NULL, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
-
-	cl_assert(expected.at_least_one_of_them_is_binary == true);
+		alien, NULL, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_dels);
+	cl_assert_equal_i(1, expected.files_binary);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_DELETED]);
 	cl_assert_equal_i(0, expected.hunks);
 	cl_assert_equal_i(0, expected.lines);
 
 	memset(&expected, 0, sizeof(expected));
 
 	cl_git_pass(git_diff_blobs(
-		NULL, alien, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
-
-	cl_assert(expected.at_least_one_of_them_is_binary == true);
+		NULL, alien, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_adds);
+	cl_assert_equal_i(1, expected.files_binary);
+	cl_assert_equal_i(1, expected.file_status[GIT_DELTA_ADDED]);
 	cl_assert_equal_i(0, expected.hunks);
 	cl_assert_equal_i(0, expected.lines);
 }
 
-static void assert_identical_blobs_comparison(diff_expects expected)
+static void assert_identical_blobs_comparison(diff_expects *expected)
 {
-	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_unmodified);
-	cl_assert_equal_i(0, expected.hunks);
-	cl_assert_equal_i(0, expected.lines);
+	cl_assert_equal_i(1, expected->files);
+	cl_assert_equal_i(1, expected->file_status[GIT_DELTA_UNMODIFIED]);
+	cl_assert_equal_i(0, expected->hunks);
+	cl_assert_equal_i(0, expected->lines);
 }
 
 void test_diff_blob__can_compare_identical_blobs(void)
 {
 	cl_git_pass(git_diff_blobs(
-		d, d, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		d, d, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
-	assert_identical_blobs_comparison(expected);
+	cl_assert_equal_i(0, expected.files_binary);
+	assert_identical_blobs_comparison(&expected);
 
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		NULL, NULL, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		NULL, NULL, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	cl_assert(expected.at_least_one_of_them_is_binary == false);
-	assert_identical_blobs_comparison(expected);
+	cl_assert_equal_i(0, expected.files_binary);
+	assert_identical_blobs_comparison(&expected);
 
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		alien, alien, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		alien, alien, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	cl_assert(expected.at_least_one_of_them_is_binary == true);
-	assert_identical_blobs_comparison(expected);
+	cl_assert(expected.files_binary > 0);
+	assert_identical_blobs_comparison(&expected);
 }
 
-static void assert_binary_blobs_comparison(diff_expects expected)
+static void assert_binary_blobs_comparison(diff_expects *expected)
 {
-	cl_assert(expected.at_least_one_of_them_is_binary == true);
+	cl_assert(expected->files_binary > 0);
 
-	cl_assert_equal_i(1, expected.files);
-	cl_assert_equal_i(1, expected.file_mods);
-	cl_assert_equal_i(0, expected.hunks);
-	cl_assert_equal_i(0, expected.lines);
+	cl_assert_equal_i(1, expected->files);
+	cl_assert_equal_i(1, expected->file_status[GIT_DELTA_MODIFIED]);
+	cl_assert_equal_i(0, expected->hunks);
+	cl_assert_equal_i(0, expected->lines);
 }
 
 void test_diff_blob__can_compare_two_binary_blobs(void)
@@ -225,16 +226,16 @@ void test_diff_blob__can_compare_two_binary_blobs(void)
 	cl_git_pass(git_blob_lookup_prefix(&heart, g_repo, &h_oid, 4));
 
 	cl_git_pass(git_diff_blobs(
-		alien, heart, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		alien, heart, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	assert_binary_blobs_comparison(expected);
+	assert_binary_blobs_comparison(&expected);
 
 	memset(&expected, 0, sizeof(expected));
 
 	cl_git_pass(git_diff_blobs(
-		heart, alien, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		heart, alien, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	assert_binary_blobs_comparison(expected);
+	assert_binary_blobs_comparison(&expected);
 
 	git_blob_free(heart);
 }
@@ -242,16 +243,16 @@ void test_diff_blob__can_compare_two_binary_blobs(void)
 void test_diff_blob__can_compare_a_binary_blob_and_a_text_blob(void)
 {
 	cl_git_pass(git_diff_blobs(
-		alien, d, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		alien, d, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	assert_binary_blobs_comparison(expected);
+	assert_binary_blobs_comparison(&expected);
 
 	memset(&expected, 0, sizeof(expected));
 
 	cl_git_pass(git_diff_blobs(
-		d, alien, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		d, alien, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
-	assert_binary_blobs_comparison(expected);
+	assert_binary_blobs_comparison(&expected);
 }
 
 /*
@@ -290,7 +291,7 @@ void test_diff_blob__comparing_two_text_blobs_honors_interhunkcontext(void)
 
 	/* Test with default inter-hunk-context (not set) => default is 0 */
 	cl_git_pass(git_diff_blobs(
-		old_d, d, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		old_d, d, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(2, expected.hunks);
 
@@ -298,7 +299,7 @@ void test_diff_blob__comparing_two_text_blobs_honors_interhunkcontext(void)
 	opts.interhunk_lines = 0;
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		old_d, d, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		old_d, d, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(2, expected.hunks);
 
@@ -306,9 +307,31 @@ void test_diff_blob__comparing_two_text_blobs_honors_interhunkcontext(void)
 	opts.interhunk_lines = 1;
 	memset(&expected, 0, sizeof(expected));
 	cl_git_pass(git_diff_blobs(
-		old_d, d, &opts, &expected, diff_file_fn, diff_hunk_fn, diff_line_fn));
+		old_d, d, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
 
 	cl_assert_equal_i(1, expected.hunks);
 
 	git_blob_free(old_d);
 }
+
+void test_diff_blob__checks_options_version_too_low(void)
+{
+	const git_error *err;
+
+	opts.version = 0;
+	cl_git_fail(git_diff_blobs(
+		d, alien, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
+	err = giterr_last();
+	cl_assert_equal_i(GITERR_INVALID, err->klass);
+}
+
+void test_diff_blob__checks_options_version_too_high(void)
+{
+	const git_error *err;
+
+	opts.version = 1024;
+	cl_git_fail(git_diff_blobs(
+		d, alien, &opts, diff_file_cb, diff_hunk_cb, diff_line_cb, &expected));
+	err = giterr_last();
+	cl_assert_equal_i(GITERR_INVALID, err->klass);
+}
