@@ -19,6 +19,8 @@ void test_repo_init__initialize(void)
 static void cleanup_repository(void *path)
 {
 	git_repository_free(_repo);
+	_repo = NULL;
+
 	cl_fixture_cleanup((const char *)path);
 }
 
@@ -302,8 +304,7 @@ void test_repo_init__sets_logAllRefUpdates_according_to_type_of_repository(void)
 
 void test_repo_init__extended_0(void)
 {
-	git_repository_init_options opts;
-	memset(&opts, 0, sizeof(opts));
+	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
 
 	/* without MKDIR this should fail */
 	cl_git_fail(git_repository_init_ext(&_repo, "extended", &opts));
@@ -325,8 +326,7 @@ void test_repo_init__extended_1(void)
 	git_reference *ref;
 	git_remote *remote;
 	struct stat st;
-	git_repository_init_options opts;
-	memset(&opts, 0, sizeof(opts));
+	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
 
 	opts.flags = GIT_REPOSITORY_INIT_MKPATH |
 		GIT_REPOSITORY_INIT_NO_DOTGIT_DIR;
@@ -351,7 +351,7 @@ void test_repo_init__extended_1(void)
 
 	cl_git_pass(git_reference_lookup(&ref, _repo, "HEAD"));
 	cl_assert(git_reference_type(ref) == GIT_REF_SYMBOLIC);
-	cl_assert_equal_s("refs/heads/development", git_reference_target(ref));
+	cl_assert_equal_s("refs/heads/development", git_reference_symbolic_target(ref));
 	git_reference_free(ref);
 
 	cl_git_pass(git_remote_load(&remote, _repo, "origin"));
@@ -365,8 +365,7 @@ void test_repo_init__extended_1(void)
 
 void test_repo_init__extended_with_template(void)
 {
-	git_repository_init_options opts;
-	memset(&opts, 0, sizeof(opts));
+	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
 
 	opts.flags = GIT_REPOSITORY_INIT_MKPATH | GIT_REPOSITORY_INIT_BARE;
 	opts.template_path = cl_fixture("template");
